﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Tempora;

namespace QuickDirty_NTPServer
{
    public partial class FormMain : Form
    {
        public static ConfigModel config = new ConfigModel();
        private static bool IsCanceled = false;
        private UdpClient global_UdpClient=null;

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            if (!Directory.Exists(Environment.CurrentDirectory + "\\Logs"))
            {
                System.IO.Directory.CreateDirectory(Environment.CurrentDirectory + "\\Logs");
            }
            if (!File.Exists(Environment.CurrentDirectory + "\\NTPServerConf.conf"))
            {
                SerializeClassHelper.SerializeObject(Environment.CurrentDirectory + "\\NTPServerConf.conf",config);
            }
            config = (ConfigModel)SerializeClassHelper.DeserializeObject(Environment.CurrentDirectory + "\\NTPServerConf.conf");
            lbl_ISStartUP.Text = config.ISStartUP.ToString();
            lbl_UDPPort.Text = config.UDPPort.ToString();
            lbl_LogCountToClear.Text = config.LogCountToClear.ToString();
            lbl_ISLogToFile.Text = config.ISLogToFile.ToString();

            if (config.ISStartUP)
            {
                btn_start_Click(sender,e);
            }

        }
        private void btn_start_Click(object sender, EventArgs e)
        {

            if (this.btn_start.Text.Equals("Start"))
            {
 
                Task.Factory.StartNew(() =>
                {
                    var udpClient = new UdpClient(config.UDPPort);
                    global_UdpClient = udpClient;
                    this.Invoke(new MethodInvoker(()=> {
                        this.btn_start.Text = "Stop";
                        this.label1.ForeColor = Color.White;
                        this.label1.BackColor = Color.Green;
                    }));
                    LogMSG("Simple NTP Server Start Successfuly. . .", true);
                    try
                    {
                        while (!IsCanceled)
                        {
                            IPEndPoint remoteEP = null;
                            var inBuffer = udpClient.Receive(ref remoteEP);
                            LogMSG(" >> "+BytesToString(inBuffer), false);

                            DateTime arrival = DateTime.Now;
                            var ntp = NtpPacket.ParseBytes(inBuffer);

                            //Thread.Sleep(16);

                            var outBuffer = NtpPacket.GetBytes(DateTime.Now, ntp, arrival);
                            udpClient.Send(outBuffer, outBuffer.Length, remoteEP);
                            LogMSG(" << " + BytesToString(outBuffer), false);
                        }
                    }catch(Exception ex)
                    {
                        LogMSG("Catch Exception：" + ex.ToString(), true);
                    }
                    finally
                    {
                        LogMSG("Simple NTP Server Close. . .", true);
                    }
                        
                        
                });

     

            }
            else if (this.btn_start.Text.Equals("Stop"))
            {
                this.Invoke(new MethodInvoker(() => {
                    this.btn_start.Text = "Start";
                    this.label1.ForeColor = Color.White;
                    this.label1.BackColor = Color.Red;
                }));
                if (global_UdpClient != null)
                {
                    global_UdpClient.Close();
                }
            }
            
            
              

        }

        private void LogMSG(string msg,bool isToFile)
        {
            this.Invoke(new MethodInvoker(()=>
            {
                if (this.rtb_log.Text.Length > config.LogCountToClear)
                {
                    this.rtb_log.Text = "";
                }
                string rz = (DateTime.Now.ToString("HH:mm:ss") + " " + msg + "\r\n");
                this.rtb_log.Text = rz + this.rtb_log.Text;

                if (isToFile && config.ISLogToFile)
                {
                    string strFilePath = Environment.CurrentDirectory + "\\Logs\\"+string.Format("{0}.txt",DateTime.Now.ToString("yyyy-MM-dd"));
                    System.IO.FileStream fs = new System.IO.FileStream(strFilePath, System.IO.FileMode.Append);
                    System.IO.StreamWriter sw = new System.IO.StreamWriter(fs, System.Text.Encoding.Default);
                    sw.Write(rz);
                    sw.Close();
                    fs.Close();
                }
            }));
        }

        private string BytesToString(byte[] bytes)
        {
            StringBuilder strBuider = new StringBuilder();
            for (int index = 0; index < bytes.Length; index++)
            {
                strBuider.Append(((int)bytes[index]).ToString("X2"));
            }
            return strBuider.ToString();
        }

        private void label2_Click(object sender, EventArgs e)
        {
            config = (ConfigModel)SerializeClassHelper.DeserializeObject(Environment.CurrentDirectory + "\\NTPServerConf.conf");
            lbl_ISStartUP.Text = config.ISStartUP.ToString();
            lbl_UDPPort.Text = config.UDPPort.ToString();
            lbl_LogCountToClear.Text = config.LogCountToClear.ToString();
            lbl_ISLogToFile.Text = config.ISLogToFile.ToString();
            LogMSG("刷新配置成功",false);
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {

            if (MessageBox.Show("你确定要退出程序吗？退出后，将不能提供时间同步服务。", "确认", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) == DialogResult.OK)
            {
                LogMSG("程序关闭。。", true);
                // 关闭所有的线程
                if (global_UdpClient != null)
                {
                    global_UdpClient.Close();
                }
                this.Dispose();
                this.Close();
               
                System.Environment.Exit(0);
            }
            else
            {
                e.Cancel = true;
            }
        }

        private void FormMain_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.Hide(); //或者是this.Visible = false;
                this.notifyIcon1.Visible = true;
            }
        }

        private void 显示ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Show();
            this.WindowState = FormWindowState.Normal;
            this.Activate();
        }

        private void 退出ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("你确定要退出程序吗？退出后，将不能提供时间同步服务。", "确认", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) == DialogResult.OK)
            {
                LogMSG("程序关闭。。", true);
                System.Environment.Exit(0);
            }
        }

        private void notifyIcon1_DoubleClick(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Normal)
            {
                this.WindowState = FormWindowState.Minimized;
                this.Hide();
            }
            else if (this.WindowState == FormWindowState.Minimized)
            {
                this.Show();
                this.WindowState = FormWindowState.Normal;
                this.Activate();
            }
        }
    }
}
